@@ -37,7 +37,7 @@
 #'             nu = 1,
 #'             Lmin = 0,
 #'             lse = TRUE,
-#'             nthreads = 4)
+#'             nthreads = 2)
 #'
 #' @param x Either a list containing the G group specific design matrices of sizes
 #' \eqn{n_i \times p_i} (general model),  a list containing the \eqn{d}
@@ -61,16 +61,14 @@
 #' \code{lambda}, as a fraction of \eqn{\lambda_{max}}; the (data dependent)
 #' smallest value for which all coefficients are zero. Used when lambda is not
 #' specified.
-#' @param lambda sequence of strictly positive floats used  as penalty parameters.
-#' @param scale_y strictly positive number that is the response \code{y} is multiplied with.
-#' @param penalty.factor array of size \eqn{p_1 \times \cdots \times p_d} of
-#' positive floats. Is multiplied with each element in \code{lambda} to allow
-#' differential penalization on the coefficients.
-#' @param reltol strictly positive float giving the convergence tolerance for
-#' the inner loop.
+#' @param lambda A sequence of strictly positive floats used  as penalty parameters.
+#' @param scale_y strictly positive number that the response \code{y} is multiplied with.
+#' @param penalty.factor a length \eqn{p} vector of positive floats that are
+#' multiplied with each element in \code{lambda} to allow differential penalization
+#' on the coefficients. For tensor models an array of size \eqn{p_1 \times \cdots \times p_d}.
+#' @param reltol strictly positive float giving the convergence tolerance.
 #' @param maxiter positive integer giving the maximum number of  iterations
-#' allowed for each \code{lambda} value, when  summing over all outer iterations
-#' for said \code{lambda}.
+#' allowed for each \code{lambda} value.
 #' @param steps strictly positive integer giving the number of steps used in the
 #' multi-step adaptive lasso algorithm for non-convex penalties. Automatically
 #' set to 1 when \code{penalty = "lasso"}.
@@ -90,7 +88,7 @@
 #' @param lse logical variable indicating whether to use the log-sum-exp-loss.  TRUE is
 #' default and yields the loss below and  FALSE yields the exponential of this.
 #' @param nthreads integer giving the number of threads to use when  openMP
-#' is available. Default is 4.
+#' is available. Default is 2.
 #' @details Consider modeling heterogeneous data \eqn{y_1,\ldots, y_n} by dividing
 #' it into \eqn{G} groups \eqn{\mathbf{y}_g = (y_1, \ldots, y_{n_g})},
 #' \eqn{g \in \{ 1,\ldots, G\}} and then using a linear model
@@ -115,7 +113,7 @@
 #' \hat V_g(\beta):=\frac{1}{n_g}(2\beta^\top \mathbf{X}_g^\top
 #' \mathbf{y}_g-\beta^\top \mathbf{X}_g^\top \mathbf{X}_g\beta),
 #' }
-#' is the empirical explained variance, see \cite{Lund et al., 2021} for more
+#' is the empirical explained variance, see \cite{Lund et al., 2022} for more
 #' details and references.
 #'
 #' The function \code{softmaximin} solves the soft maximin estimation problem in
@@ -146,8 +144,9 @@
 #' arithmetic, see also \code{\link{RH}}.
 #' ii) second, if the design matrix \eqn{\mathbf{X}} is the inverse matrix of an
 #' orthogonal wavelet transform \code{softmaximin}  solves the soft maximin problem
-#'  provided only with the \eqn{d + 1} dimensional response array \code{y}
-#'  (i.e. \code{x = NULL}) and uses the  pyramid algorithm to compute multiplications
+#'  given the \eqn{d + 1} dimensional response array \code{y} and
+#'    \code{x} the name of the wavelet family \code{\link{wt}},  using the
+#'    pyramid algorithm to compute multiplications
 #'  involving \eqn{\mathbf{X}}.
 #'
 #' Note that when multiple values for \eqn{\zeta} is provided it is  possible to
@@ -169,7 +168,8 @@
 #' For array data a vector giving the dimension of the model coefficient array \eqn{\beta}.}
 #' \item{dimobs}{An integer giving the number of observations. For array data a
 #' vector giving the dimension of the observation (response) array \code{Y}.}
-#' \item{dim}{A string indicating the wavelet name if used.}
+#' \item{dim}{Integer indicating the dimension of of the array model. Equal to 1
+#' for non array.}
 #' \item{wf}{A string indicating the wavelet name if used.}
 #' \item{diagnostics}{A list of length 3. Item \code{iter} is a \code{length(zeta)}-list
 #' of vectors containing  the number of   iterations for each \code{lambda} value
@@ -187,7 +187,8 @@
 #'
 #' @references
 #' Lund, A., S. W. Mogensen and N. R. Hansen (2022). Soft Maximin Estimation for
-#' Heterogeneous Data. \emph{Scandinvaian Journal of Statistics}.
+#' Heterogeneous Data. \emph{Scandinavian Journal of Statistics}, vol. 49, no. 4,
+#' pp. 1761-1790.
 #' url = {https://doi.org/10.1111/sjos.12580}
 #'
 #' @keywords package
@@ -219,7 +220,8 @@
 #' betahat <- fit$coef
 #'
 #' ##estimated common effects for specific lambda and zeta
-#' modelno <- 9; zetano <- 2
+#' zetano <- 2
+#' modelno <- dim(betahat[[zetano]])[2]
 #' m <- min(betahat[[zetano]][ , modelno], common_effects)
 #' M <- max(betahat[[zetano]][ , modelno], common_effects)
 #' plot(common_effects, type = "p", ylim = c(m, M), col = "red")
@@ -250,14 +252,15 @@
 #' ##fit model for range of lambda and zeta
 #' system.time(fit <- softmaximin(x, y, zeta = c(1, 10, 100), penalty = "lasso",
 #'             alg = "npg"))
-#' Betahat <- fit$coef
+#' betahat <- fit$coef
 #'
 #' ##estimated common effects for specific lambda and zeta
-#' modelno <- 30; zetano <- 1
-#' m <- min(Betahat[[zetano]][, modelno], common_effects)
-#' M <- max(Betahat[[zetano]][, modelno], common_effects)
-#' plot(common_effects, type = "h", ylim = c(m, M), col = "red")
-#' lines(Betahat[[zetano]][, modelno], type = "h")
+#' zetano <- 1
+#' modelno <- dim(betahat[[zetano]])[2]
+#' m <- min(betahat[[zetano]][, modelno], common_effects)
+#' M <- max(betahat[[zetano]][, modelno], common_effects)
+#' plot(common_effects, type = "p", ylim = c(m, M), col = "red")
+#' lines(betahat[[zetano]][ , modelno], type = "h")
 #'
 #' #Array data and wavelets
 #' ##size of example
@@ -280,21 +283,22 @@
 #' ##fit model for range of lambda and zeta
 #' system.time(fit <- softmaximin(x = "la8", y, zeta = c(0.1, 1, 10),
 #'                                 penalty = "lasso", alg = "fista"))
-#' Betahat <- fit$coef
+#' betahat <- fit$coef
 #'
 #' ##estimated common effects for specific lambda and zeta
-#' modelno <- 10; zetano <- 3
-#' m <- min(Betahat[[zetano]][, modelno], common_effects)
-#' M <- max(Betahat[[zetano]][, modelno], common_effects)
-#' plot(common_effects, type = "h", ylim = c(m, M), col = "red")
-#' lines(Betahat[[zetano]][, modelno], type = "h")
-#'
+#' zetano <- 3
+#' modelno <- dim(betahat[[zetano]])[2]
+#' m <- min(betahat[[zetano]][, modelno], common_effects)
+#' M <- max(betahat[[zetano]][, modelno], common_effects)
+#' plot(common_effects, type = "p", ylim = c(m, M), col = "red")
+#' lines(betahat[[zetano]][ , modelno], type = "h")
+
 #' @export
 #' @useDynLib SMME, .registration = TRUE
 #' @importFrom Rcpp evalCpp
 softmaximin <- function(
-               x, #G list or d list or string
-               y, #G list or d array
+               x,
+               y,
                zeta,
                penalty = c("lasso", "scad"),
                alg = c("npg", "fista"),
@@ -313,7 +317,7 @@ softmaximin <- function(
                nu = 1,
                Lmin = 0,
                lse = TRUE,
-               nthreads = 4){
+               nthreads = 2){
 wf = "not used"
 wave = 0
 J = 0
@@ -355,10 +359,15 @@ if("list" %in% class(y)){##general
   n <- sum(sapply(Z, length))
   p <- dim(x[[1]])[2]
   alg = 1 #npg
-  if(is.null(penalty.factor)){penalty.factor <- as.matrix(rep(1, p))
-  }else{penalty.factor <- as.matrix(penalty.factor)}
+  if(is.null(penalty.factor)){
+    penalty.factor <- as.matrix(rep(1, p))
+  }else{
+    penalty.factor <- as.matrix(penalty.factor)
+
+    }
   #check to make sure y is compaitble with x in every gropu...todo
-}else if("array" %in% class(y)){#array
+
+  }else if("array" %in% class(y)){#array
 
 array = 1
 
@@ -436,7 +445,25 @@ for(i in 1:G){
 
 }
 
-if(is.null(penalty.factor)){penalty.factor <- matrix(1, p1, p2 * p3)}
+#if(is.null(penalty.factor)){penalty.factor <- matrix(1, p1, p2 * p3)}
+
+if(is.null(penalty.factor)){
+
+  penalty.factor <- matrix(1, p1, p2 * p3)
+
+}else if(length(penalty.factor) != p){
+
+  stop(
+    paste("number of elements in penaltyfactor (", length(penalty.factor),") is not equal to the number of coefficients (", p,")", sep = "")
+  )
+
+}else {
+
+  if(min(penalty.factor) < 0){stop(paste("penaltyfactor must be positive"))}
+
+  penalty.factor <- matrix(penalty.factor, p1, p2 * p3)
+
+}
 
 }else{
   stop(paste("response y must be list or array"))
