@@ -5,6 +5,7 @@
 #include "RcppArmadillo.h" //for arma namespace?
 
 #include "auxfunc.h"
+#include "wave_filter.cpp"
 
 using namespace std;
 using namespace arma;
@@ -34,8 +35,11 @@ Rcpp::List pga(Rcpp::List phi,
                int array,
                int ll,
                double Lmin,
-               int nthreads){
-
+               int nthreads,
+               int wave,
+               int J,
+               int dim,
+               std::string wf){
 
 Rcpp::List output, Resp(resp), Phi(phi);
 int G = Resp.size();
@@ -45,13 +49,22 @@ if(array == 1){//npg and fista for array data------
 arma::mat Phi1 = Rcpp::as<arma::mat>(Phi[0]);
 arma::mat Phi2 = Rcpp::as<arma::mat>(Phi[1]);
 arma::mat Phi3 = Rcpp::as<arma::mat>(Phi[2]);
-arma::cube Z(Phi1.n_rows, Phi2.n_rows * Phi3.n_rows, G);
 
-for(int i = 0; i < G; i++){Z.slice(i) = Rcpp::as<arma::mat>(Resp[i]);}
+double *g = nullptr, *h = nullptr;
 
-int n1 = Phi1.n_rows, n2 = Phi2.n_rows, n3 = Phi3.n_rows, ng = n1 * n2 * n3,
-    p1 = Phi1.n_cols, p2 = Phi2.n_cols, p3 = Phi3.n_cols, p = p1 * p2 * p3,
-    nzeta = zeta.n_elem;
+int Lwave, n1, n2, n3, p1, p2, p3;
+if(wave == 1){
+  Lwave = get_L(wf);
+  g = new double[Lwave];
+  h = new double[Lwave];
+  wave_filter(wf, g, h, Lwave);
+n1 = Phi1(0, 0), n2 = Phi2(0, 0), n3 = Phi3(0, 0), p1 = n1, p2 = n2, p3 = n3;
+}else{
+n1 = Phi1.n_rows, n2 = Phi2.n_rows, n3 = Phi3.n_rows,
+p1 = Phi1.n_cols, p2 = Phi2.n_cols, p3 = Phi3.n_cols;
+}
+
+int  ng = n1 * n2 * n3,     p = p1 * p2 * p3,    nzeta = zeta.n_elem;
 
 double alphamax, delta, deltamax, L;
 
@@ -60,75 +73,67 @@ arma::vec Btenter(nzeta),  Btiter(nzeta),  eig1, eig2, eig3,  EndMod(nzeta);
 arma::mat DF(nlambda, nzeta), ITER(nlambda, nzeta), Lamb(nlambda, nzeta),
           Phi1tPhi1, Phi2tPhi2, Phi3tPhi3, Stops(3, nzeta);
 
-arma::cube Coef(p, nlambda, nzeta), OBJ(maxiter, nlambda, nzeta), PhitZ(p1, p2 * p3, G);
+arma::cube Coef(p, nlambda, nzeta), OBJ(maxiter, nlambda, nzeta),
+           PhitZ(p1, p2 * p3, G),    Z(n1, n2 * n3, G);
 
+for(int i = 0; i < G; i++){Z.slice(i) = Rcpp::as<arma::mat>(Resp[i]);}
 
-////precompute
+if(wave == 1){alphamax = 1;}else{
 Phi1tPhi1 = Phi1.t() * Phi1;
 Phi2tPhi2 = Phi2.t() * Phi2;
 Phi3tPhi3 = Phi3.t() * Phi3;
 eig1 = arma::eig_sym(Phi1tPhi1);
 eig2 = arma::eig_sym(Phi2tPhi2);
 eig3 = arma::eig_sym(Phi3tPhi3);
-alphamax = as_scalar(max(kron(eig1, kron(eig2 , eig3))));
-
+alphamax = as_scalar(max(kron(eig1, kron(eig2, eig3))));
+}
 for(int j = 0; j < G; j++){
-
-  PhitZ.slice(j) = RHmat(Phi3.t(), RHmat(Phi2.t(),
-                         RHmat(Phi1.t(), Z.slice(j), n2, n3), n3, p1), p1, p2);
-
+if(wave == 1){
+PhitZ.slice(j) = wt(Z.slice(j), dim, Lwave, h, g, J, p1, p2, p3, PhitZ.slice(j));
+}else{
+PhitZ.slice(j) = RHmat(Phi3.t(), RHmat(Phi2.t(),
+RHmat(Phi1.t(), Z.slice(j), n2, n3), n3, p1), p1, p2);
+}
 }
-
-////proximal step size
-// Sumsqdiff.fill(0);
-// for(int i = 0; i < G; i++){
-// Zi = Z.slice(i);
-// for(int j = i + 1; j < G; j++){Sumsqdiff(i,j) = sum_square(Zi - Z.slice(j));}
-// }
 
 mat A(G, G);
 mat PhitZi;
 A.fill(0);
 for(int i = 0; i < G; i++){
-  PhitZi = PhitZ.slice(i);
-  for(int j = i + 1; j < G; j++){
-    A(i,j) = sum_square(PhitZi- PhitZ.slice(j));
-  }
+PhitZi = PhitZ.slice(i);
+for(int j = i + 1; j < G; j++){
+A(i,j) = sum_square(PhitZi- PhitZ.slice(j));
+}
 }
 
 L = 4 / pow(ng, 2) * (max(max(A)) + alphamax * ng / 2); //upper bound on Lipschitz constant
-//L =4 * alphamax / pow(ng, 2) * (max(max(Sumsqdiff)) + ng / 2); //upper bound on Lipschitz constant
-delta = nu * 1.9 / L; //stepsize scaled up bynu
+delta = nu * 1.9 / L; //stepsize scaled up by nu
 deltamax = 1.99 / L; //maximum theoretically allowed stepsize
 if(Lmin == 0){Lmin = (1 / nu) * L;}
 
 ////make lambda sequence, Beta is ZERO here so lambdamax identical across zeta
 if(makelamb == 1){
 
- // arma::mat Ze = zeros<mat>(n1, n2 * n3);
-  arma::mat absgradzeroall(p1, p2 * p2);
+arma::mat absgradzeroall(p1, p2 * p2);
+absgradzeroall = abs(gradloss(PhitZ, arma::zeros<mat>(p1, p2 * p3),
+                              -eev(arma::zeros<mat>(n1, n2 * n3), Z, ng), ng, 42, ll));
+arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
+arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
+double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
+double m = log(lambdaminratio);
+double M = 0;
+double difflamb = abs(M - m) / (nlambda - 1);
+double l = 0;
 
-  absgradzeroall = abs(gradloss(PhitZ, arma::zeros<mat>(p1, p2 * p3),
-                                -eev(arma::zeros<mat>(n1, n2 * n3), Z, ng), ng, 42, ll));
+for(int i = 0; i < nlambda ; i++){
 
-  arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
-  arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
-  double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
-  double m = log(lambdaminratio);
-  double M = 0;
-  double difflamb = abs(M - m) / (nlambda - 1);
-  double l = 0;
+lambda(i) = lambdamax * exp(l);
+l = l - difflamb;
 
-  for(int i = 0; i < nlambda ; i++){
-
-    lambda(i) = lambdamax * exp(l);
-    l = l - difflamb;
-
-  }
+}
 
 }else{std::sort(lambda.begin(), lambda.end(), std::greater<int>());}
 
-
 #ifdef _OPENMP
 openmp = 1;
 #pragma omp parallel num_threads(nthreads)
@@ -141,21 +146,21 @@ openmp = 1;
 for(int z = 0; z < nzeta ; z++){
 
 int ascent, ascentmax, bt, btenter = 0, btiter = 0, endmodelno = nlambda,
-    Stopconv = 0, Stopmaxiter = 0, Stopbt = 0;
+Stopconv = 0, Stopmaxiter = 0, Stopbt = 0;
 
 double ascad = 3.7, lossBeta, lossProp = 0, lossX, penProp = 0, relobj, val;
 
 arma::vec df(nlambda), Iter(nlambda), Pen(maxiter),
-          obj(maxiter + 1),  eevBeta, eevProp, eevX;
+obj(maxiter + 1),  eevBeta, eevProp, eevX;
 
 arma::mat absBeta(p1, p2 * p3), Beta(p1, p2 * p3), Betaprev(p1, p2 * p3),
-          Betas(p, nlambda), BT(nlambda, maxiter), Delta(maxiter, nlambda),
-            dpen(p1, p2 * p3), Gamma(p1, p2 * p3),
-          GradlossX(p1, p2 * p3), GradlossXprev(p1, p2 * p3), GradlossX2(p1, p2 * p3),
-           Obj(maxiter, nlambda),
-           PhitPhiBeta, PhitPhiX, pospart(p1, p2 * p3),
-          Prop(p1, p2 * p3), PhiBeta(n1, n2 * n3), PhiProp(n1, n2 * n3), PhiX(n1, n2 * n3),
-          wGamma(p1, p2 * p3), R, S,  X(p1, p2 * p3), Xprev, Zi;
+Betas(p, nlambda), BT(nlambda, maxiter), Delta(maxiter, nlambda),
+dpen(p1, p2 * p3), Gamma(p1, p2 * p3),
+GradlossX(p1, p2 * p3), GradlossXprev(p1, p2 * p3), GradlossX2(p1, p2 * p3),
+Obj(maxiter, nlambda),
+PhitPhiBeta, PhitPhiX, pospart(p1, p2 * p3),
+Prop(p1, p2 * p3), PhiBeta(n1, n2 * n3), PhiProp(n1, n2 * n3), PhiX(n1, n2 * n3),
+wGamma(p1, p2 * p3), R, S,  X(p1, p2 * p3), Xprev, Zi;
 
 ////fill variables
 ascentmax = 4;
@@ -176,44 +181,28 @@ Betaprev.fill(0);
 Beta = Betaprev;
 X = Beta; //npg only uses X
 Xprev = Betaprev; //npg only uses X
+
+if(wave == 1){//idwt!!
+PhiBeta = iwt(Beta, dim, Lwave, h, g, J, p1, p2, p3, PhiBeta);
+}else{
 PhiBeta = RHmat(Phi3, RHmat(Phi2, RHmat(Phi1, Beta, p2, p3), p3, n1), n1, n2);
+}
 PhiX = PhiBeta; //npg only uses X
-PhitPhiBeta = RHmat(Phi3tPhi3, RHmat(Phi2tPhi2, RHmat(Phi1tPhi1, Beta, p2, p3), p3, p1), p1, p2);
+if(wave == 1){//identity!!
+PhitPhiBeta = Beta;
+}else{
+PhitPhiBeta = RHmat(Phi3tPhi3,
+RHmat(Phi2tPhi2,
+RHmat(Phi1tPhi1, Beta, p2, p3), p3, p1), p1, p2);
+}
+
 PhitPhiX = PhitPhiBeta; //npg only uses X
 eevBeta = -eev(PhiBeta, Z, ng);
 eevX = eevBeta;
 
-//zeta loop
-// for(int z = 0; z < nzeta ; z++){
-
 lossBeta = softmaxloss(eevBeta, zeta(z), ll);
 lossX = lossBeta; //npg only uses X
 
-// ////make lambda sequence, NOTE Beta is set to ZERO here!!
-// if(makelamb == 1){
-//
-// //arma::mat Ze = zeros<mat>(n1, n2 * n3);
-// arma::mat absgradzeroall(p1, p2 * p2);
-//
-// absgradzeroall = abs(gradloss(PhitZ, PhitPhiBeta, -eev(PhiBeta, Z, ng), ng, zeta(z), ll));
-//
-// arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
-// arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
-// double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
-// double m = log(lambdaminratio);
-// double M = 0;
-// double difflamb = abs(M - m) / (nlambda - 1);
-// double l = 0;
-//
-// for(int i = 0; i < nlambda ; i++){
-//
-// lambda(i) = lambdamax * exp(l);
-// l = l - difflamb;
-//
-// }
-//
-// }else{std::sort(lambda.begin(), lambda.end(), std::greater<int>());}
-
 ///////////start lambda loop
 for (int j = 0; j < nlambda; j++){
 
@@ -240,7 +229,6 @@ wGamma = abs(dpen) % Gamma / lambda(j) % (Beta != 0) + lambda(j) * (Beta == 0);
 }
 
 }
-
 if(alg == 1){/////////////////ARRAY NPG algorithm from chen2016
 
 for (int k = 0; k < maxiter; k++){
@@ -254,7 +242,14 @@ Delta(k, j) = delta;
 
 }else{//if not the first iteration
 
-PhitPhiX = RHmat(Phi3tPhi3, RHmat(Phi2tPhi2, RHmat(Phi1tPhi1, X, p2, p3), p3, p1), p1, p2);
+if(wave == 1){//identity!!
+PhitPhiX = X;
+}else{
+PhitPhiX = RHmat(Phi3tPhi3,
+RHmat(Phi2tPhi2,
+RHmat(Phi1tPhi1, X, p2, p3), p3, p1), p1, p2);
+}
+
 GradlossX = gradloss(PhitZ, PhitPhiX, eevX, ng, zeta(z), ll);
 lossX = softmaxloss(eevX, zeta(z), ll);
 
@@ -267,12 +262,18 @@ delta = 1 / std::min(tmp, pow(10,8));
 }else{
 delta = 1;
 }
+
 ////proximal backtracking from chen2016
 BT(j, k) = 0;
 while(BT(j, k) < btmax){
 
 Prop = prox_l1(X - delta * GradlossX, delta * wGamma);
+if(wave == 1){//idwt!!
+PhiProp = iwt(Prop, dim, Lwave, h, g, J, p1, p2, p3, PhiProp);
+}else{
 PhiProp = RHmat(Phi3, RHmat(Phi2, RHmat(Phi1, Prop, p2, p3), p3, n1), n1, n2);
+}
+
 eevProp = -eev(PhiProp, Z, ng);
 lossProp = softmaxloss(eevProp, zeta(z), ll);
 
@@ -300,10 +301,7 @@ GradlossXprev = GradlossX;
 X = Prop;
 PhiX = PhiProp;
 eevX = eevProp;
-//lossX = lossProp;
-//penX = penProp;
 obj(k) = lossProp + penProp;
-//Pen(k) = penX;
 Obj(k, j) = obj(k);
 Iter(j) = k;
 Delta(k, j) = delta;
@@ -358,9 +356,19 @@ Delta(k, j) = delta;
 
 X = Beta + (k - 2) / (k + 1) * (Beta - Betaprev);
 
+if(wave == 1){
+PhiX = iwt(X, dim, Lwave, h, g, J, p1, p2, p3, PhiX);
+}else{
 PhiX = RHmat(Phi3, RHmat(Phi2, RHmat(Phi1, X, p2, p3), p3, n1), n1, n2);
+}
+
 eevX = -eev(PhiX, Z, ng);
+if(wave == 1){
+PhitPhiX = X;
+}else{
 PhitPhiX = RHmat(Phi3tPhi3, RHmat(Phi2tPhi2, RHmat(Phi1tPhi1, X, p2, p3), p3, p1), p1, p2);
+}
+
 GradlossX = gradloss(PhitZ, PhitPhiX, eevX, ng, zeta(z), ll);
 
 ////check if proximal backtracking occurred last iteration
@@ -377,7 +385,13 @@ BT(j, k) = 0;
 while(BT(j, k) < btmax){//start backtracking
 
 Prop = prox_l1(X - delta * GradlossX, delta * wGamma);
+
+if(wave == 1){
+PhiProp = iwt(Prop, dim, Lwave, h, g, J, p1, p2, p3, PhiProp);
+}else{
 PhiProp = RHmat(Phi3, RHmat(Phi2, RHmat(Phi1, Prop, p2, p3), p3, n1), n1, n2);
+}
+
 lossProp = softmaxloss(-eev(PhiProp, Z, ng), zeta(z), ll);
 
 val = as_scalar(lossX + accu(GradlossX % (Prop - X))
@@ -397,19 +411,22 @@ BT(j, k) = BT(j, k) + 1;
 }
 
 }//end backtracking
- ////check if maximum number of proximal backtraking step is reached
+////check if maximum number of proximal backtraking step is reached
 if(BT(j, k) == btmax){Stopbt = 1;}
 
 }else{//no backtracking
 
 Prop = prox_l1(X - delta * GradlossX, delta * wGamma);
+if(wave == 1){
+PhiProp = iwt(Prop, dim, Lwave, h, g, J, p1, p2, p3, PhiProp);
+}else{
 PhiProp = RHmat(Phi3, RHmat(Phi2, RHmat(Phi1, Prop, p2, p3), p3, n1), n1, n2);
+}
+
 lossProp = softmaxloss(-eev(PhiProp, Z, ng), zeta(z), ll);
 
 }
 
-
-
 Betaprev = Beta;
 Beta = Prop;
 lossBeta = lossProp;
@@ -418,7 +435,6 @@ Iter(j) = k;
 Delta(k, j) = delta;
 Obj(k, j) = obj(k);
 
-
 ////proximal divergence check
 if(obj(k) > obj(k - 1)){ascent = ascent + 1;}else{ascent = 0;}
 
@@ -476,8 +492,6 @@ break;
 
 }//end lambda loop
 
-
-
 Stops(0, z) = Stopconv;
 Stops(1, z) = Stopmaxiter;
 Stops(2, z) = Stopbt;
@@ -499,17 +513,16 @@ Lamb.col(z) = lambda;
 }
 #endif
 output = Rcpp::List::create(Rcpp::Named("Beta") = Coef,
-                            Rcpp::Named("df") = DF,
-                            Rcpp::Named("btenter") = Btenter,
-                            Rcpp::Named("btiter") = Btiter,
-                            Rcpp::Named("Obj") = OBJ,
-                            Rcpp::Named("Iter") = ITER,
-                            Rcpp::Named("endmodelno") = EndMod,
-                            Rcpp::Named("lambda") = Lamb,
-                          //  Rcpp::Named("BT") = BT,
-                            Rcpp::Named("Stops") = Stops,
-                            Rcpp::Named("openMP") = openmp
-                              );
+Rcpp::Named("df") = DF,
+Rcpp::Named("btenter") = Btenter,
+Rcpp::Named("btiter") = Btiter,
+Rcpp::Named("Obj") = OBJ,
+Rcpp::Named("Iter") = ITER,
+Rcpp::Named("endmodelno") = EndMod,
+Rcpp::Named("lambda") = Lamb,
+Rcpp::Named("Stops") = Stops,
+Rcpp::Named("openMP") = openmp
+);
 
 }else{//non array
 
@@ -537,31 +550,31 @@ cube Coef(p, nlambda, nzeta), OBJ(maxiter, nlambda, nzeta);
 //make lambda sequence
 if(makelamb == 1){
 
-  arma::mat absgradzeroall(p, 1);
+arma::mat absgradzeroall(p, 1);
 
-  absgradzeroall = abs(gradloss_f(PHItRESP, cube_mult(PHItPHI, arma::zeros<mat>(p, 1)),
-                                  -eev_f(field_mult(PHI, arma::zeros<mat>(p, 1)), RESP, n), n, 42, ll));
+absgradzeroall = abs(gradloss_f(PHItRESP, cube_mult(PHItPHI, arma::zeros<mat>(p, 1)),
+-eev_f(field_mult(PHI, arma::zeros<mat>(p, 1)), RESP, n), n, 42, ll));
 
-  arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
-  arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
-  double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
-  double m = log(lambdaminratio);
-  double M = 0;
-  double difflamb = abs(M - m) / (nlambda - 1);
-  double l = 0;
+arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
+arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
+double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
+double m = log(lambdaminratio);
+double M = 0;
+double difflamb = abs(M - m) / (nlambda - 1);
+double l = 0;
 
-  for(int i = 0; i < nlambda ; i++){
+for(int i = 0; i < nlambda ; i++){
 
-    lambda(i) = lambdamax * exp(l);
-    l = l - difflamb;
+lambda(i) = lambdamax * exp(l);
+l = l - difflamb;
 
-  }
+}
 
 }else{std::sort(lambda.begin(), lambda.end(), std::greater<int>());}
 
 #ifdef _OPENMP
 openmp = 1;
-#pragma omp parallel num_threads(nthreads)
+#pragma omp parallel num_threads(nthreads) //valgrind doesnt like this!!
 {
 #endif
 
@@ -575,13 +588,13 @@ int btiter = 0, endmodelno = nlambda, Stopconv = 0, Stopmaxiter = 0, Stopbt = 0;
 double ascad = 3.7, delta, lossProp = 0, lossX, penProp = 0, relobj, val;
 
 arma::vec df(nlambda), eevBeta, eevProp, eevX, Iter(nlambda), obj(maxiter + 1),
-          Pen(maxiter);
+Pen(maxiter);
 
 arma::mat absX(p, 1), Betas(p, nlambda), BT(nlambda, maxiter),
-          Delta(maxiter, nlambda), dpen(p, 1), Gamma(p, 1), GradlossX(p, 1),
-          GradlossXprev(p, 1), GradlossX2(p, 1), Obj(maxiter, nlambda), PHItPHIX,
-          pospart(p, 1), Prop(p, 1),
-          wGamma(p, 1), R,S,X(p, 1), Xprev(p, 1);
+Delta(maxiter, nlambda), dpen(p, 1), Gamma(p, 1), GradlossX(p, 1),
+GradlossXprev(p, 1), GradlossX2(p, 1), Obj(maxiter, nlambda), PHItPHIX,
+pospart(p, 1), Prop(p, 1),
+wGamma(p, 1), R,S,X(p, 1), Xprev(p, 1);
 
 field<mat> PHIX, PHIProp;
 
@@ -596,45 +609,39 @@ Delta.fill(NA_REAL);//todo
 Obj.fill(NA_REAL);//todo
 Pen.fill(0);//todo
 
-//Lmin = (1 / nu) * L;....//todo
-
 ////initialize at zero which is optimal for lambmax pr construction
 Xprev.fill(0);
 X = Xprev;
 PHIX = field_mult(PHI, X); // G * n_g field
 PHItPHIX = cube_mult(PHItPHI, X); // p x G matrix
 eevX = -eev_f(PHIX, RESP, n);
-
-
 lossX = softmaxloss(eevX, zeta(z), ll);
 
-//
-// //make lambda sequence
-// if(makelamb == 1){
-//
-// //arma::vec Ze = zeros<vec>(n);
-// arma::mat absgradzeroall(p, 1);
-//
-// absgradzeroall = abs(gradloss_f(PHItRESP, PHItPHIX, eevX, n, zeta(z), ll)); //todo gradloss
-//
-// arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
-// arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
-// double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
-// double m = log(lambdaminratio);
-// double M = 0;
-// double difflamb = abs(M - m) / (nlambda - 1);
-// double l = 0;
-//
-// for(int i = 0; i < nlambda ; i++){
-//
-// lambda(i) = lambdamax * exp(l);
-// l = l - difflamb;
-//
-// }
-//
-// }else{std::sort(lambda.begin(), lambda.end(), std::greater<int>());}
+//make lambda sequence
+if(makelamb == 1){
 
-/////////start lambda loop
+arma::mat absgradzeroall(p, 1);
+
+absgradzeroall = abs(gradloss_f(PHItRESP, PHItPHIX, eevX, n, zeta(z), ll)); //todo gradloss
+
+arma::mat absgradzeropencoef = absgradzeroall % (penaltyfactor > 0);
+arma::mat penaltyfactorpencoef = (penaltyfactor == 0) * 1 + penaltyfactor;
+double lambdamax = as_scalar(max(max(absgradzeropencoef / penaltyfactorpencoef)));
+double m = log(lambdaminratio);
+double M = 0;
+double difflamb = abs(M - m) / (nlambda - 1);
+double l = 0;
+
+for(int i = 0; i < nlambda ; i++){
+
+lambda(i) = lambdamax * exp(l);
+l = l - difflamb;
+
+}
+
+}else{std::sort(lambda.begin(), lambda.end(), std::greater<int>());}
+
+///////start lambda loop
 for (int j = 0; j < nlambda; j++){
 
 Gamma = penaltyfactor * lambda(j);
@@ -793,20 +800,46 @@ Lamb.col(z) = lambda;
 }
 #endif
 
-output = Rcpp::List::create(Rcpp::Named("Beta") = Coef,
-                            Rcpp::Named("df") = DF,
-                          //  Rcpp::Named("btenter") = Btenter,
-                            Rcpp::Named("btiter") = Btiter,
-                            Rcpp::Named("Obj") = OBJ,
-                            Rcpp::Named("Iter") = ITER,
-                            Rcpp::Named("endmodelno") = EndMod,
-                            Rcpp::Named("lambda") = Lamb,
-                            //  Rcpp::Named("BT") = BT,
-                            Rcpp::Named("Stops") = Stops,
-                              Rcpp::Named("openMP") = openmp
+output = Rcpp::List::create(
+Rcpp::Named("Beta") = Coef,
+Rcpp::Named("df") = DF,
+//  Rcpp::Named("btenter") = Btenter,
+Rcpp::Named("btiter") = Btiter,
+//  Rcpp::Named("Obj") = OBJ,
+Rcpp::Named("Iter") = ITER,
+Rcpp::Named("endmodelno") = EndMod,
+Rcpp::Named("lambda") = Lamb,
+//  Rcpp::Named("BT") = BT,
+Rcpp::Named("Stops") = Stops,
+Rcpp::Named("openMP") = openmp
 );
 }
 
 return output;
 
 }
+
+//[[Rcpp::export]]
+arma::mat WT(arma::mat x, int dim, std::string wf, int J, int p1, int p2, int p3){
+arma::mat out(p1, p2 * p3);
+  int L = get_L(wf);
+  double *g = nullptr, *h = nullptr;
+  g = new double[L];
+  h = new double[L];
+  wave_filter(wf, g, h, L);
+out = wt(x, dim, L, h, g, J, p1, p2, p3, out);
+return out;
+}
+
+//[[Rcpp::export]]
+arma::mat IWT(arma::mat x, int dim, std::string wf, int J, int p1, int p2, int p3){
+arma::mat out(p1, p2 * p3);
+  int L = get_L(wf);
+  double *g = nullptr, *h = nullptr;
+  g = new double[L];
+  h = new double[L];
+  wave_filter(wf, g, h, L);
+out =   iwt(x, dim, L,  h, g,   J,   p1,   p2,   p3, out);
+return out;
+
+}
